<?php


namespace Drupal\mediaflow\Element;


use Drupal;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\FormElement as FormElementBase;
use Drupal\Core\Render\Annotation\FormElement;
use Drupal\Component\Serialization\Json;
use Drupal\mediaflow\Service\MediaflowFetcher;

/**
 * Provides a media flow resource selector form element.
 *
 * Usage example:
 * @code
 * $build = [
 *   '#type' => 'mediaflow_selector_file',
 * ];
 * @endcode
 *
 * @FormElement("mediaflow_selector_file")
 */
class MediaflowSelectorFile extends FormElementBase {

  /**
   */
  public function getInfo() {
    $class = get_class($this);

    return [
      '#process' => [
        [$class, 'processMediaflowSelector'],
      ],
      '#element_validate' => [
        [$class, 'validateMediaflowSelector'],
      ]
      
    ];
  }

  /**
   * @param array $element
   * @param FormStateInterface $form_state
   * @param array $complete_form
   */
  public static function processMediaflowSelector(array &$element, FormStateInterface $form_state, array &$complete_form) {

    $settings = Drupal::config('mediaflow.settings');
    $element['#tree'] = true;

    $element['value'] = [
      '#type' => 'hidden',
      '#attributes' => [
        'data-mediaflow-target' => TRUE,
      ]
    ];

    if (is_numeric($element['#value'])) {
      $element['value']['#default_value'] = $element['#value'];
    } else if(strlen($element['#value']) == 0){
      $element['value']['#default_value'] = "0";
    }

    $limit_file_type = 'image,video,srt,vtt';
    if (!empty($element['#media_type'])) {
      $limit_file_type = $element['#media_type'];
    }

    $api_settings = [
      'client_id'=> MediaflowFetcher::CLIENT_ID,
      'client_secret'=> MediaflowFetcher::CLIENT_SECRET,
      'refresh_token'=> $settings->get('refresh_token'),
      'allowSelectFormat' => $settings->get('allow_select_format'),
      'setAltText' => $settings->get('set_alt_text'),
      'locale' => Drupal::currentUser()->getPreferredAdminLangcode(),
      'limitFileType' => $limit_file_type,
    ];

    if(!empty($element['#value'])){
      $element['mediaflowbrowser'] = [
        '#type' => 'markup',     
        '#markup' => '<h5>'.$element['#title'].'</h5><span class="button" data-drupal-selector="edit-submit" id="pick-mediaflow-button">Pick Mediaflow media</span><span id="remove-mediaflow-button" class="button button--danger">Remove media</span><div id="mediaflow-file" style="position:relative;width:1000px;height:500px;overflow:hidden;border:1px solid black;margin:10px; display: none;" data-mediaflow-selector-file></div>',
        '#attached' => [
          'library' => ['mediaflow/element.selector'],
          'drupalSettings' => ['mediaflow' => $api_settings],
        ],
      ];
    } else {
      $element['mediaflowbrowser'] = [
        '#type' => 'markup',     
        '#markup' => '<h5>'.$element['#title'].'</h5><span class="button" data-drupal-selector="edit-submit" id="pick-mediaflow-button">Pick Mediaflow media</span><div id="mediaflow-file" style="position:relative;width:1000px;height:500px;overflow:hidden;border:1px solid black;margin:10px; display: none;" data-mediaflow-selector-file></div>',
        '#attached' => [
          'library' => ['mediaflow/element.selector'],
          'drupalSettings' => ['mediaflow' => $api_settings],
        ],
      ];
   
    }  

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function valueCallback(&$element, $input, FormStateInterface $form_state) {
    if ($input === FALSE) {
      $element += ['#default_value' => ''];
      return $element['#default_value'];
    }
    if (!empty($input['value']) && $input['value'] != "0") {
      $data = Json::decode($input['value']);
      if (is_object($data))
      { 
        $returnData = Drupal::service('mediaflow.usage_manager')->addMedia($data);

        return $returnData;
      }
    } else if($input['value'] == "0"){ /*solution to reset field*/
      return "";
    }

    return "";
  }

  public static function validateMediaflowSelector(&$element, FormStateInterface $form_state, &$complete_form) {
    $input = $form_state->getValue($element['#parents']);
    var_dump($input['value']);

    //check to see if old id, new json or reset
   if (!empty($input['value']) && $input['value'] != "0") {
      if (is_numeric($input['value'])) {
        $form_state->setValueForElement($element, $input['value']);
      } else {
        $data = Json::decode($input['value']);
        $mediaflow_id = Drupal::service('mediaflow.usage_manager')->addMedia($data);
      
        if (!empty($mediaflow_id)) {
          $form_state->setValueForElement($element, $mediaflow_id);
        }
        else {
          // @todo Error message
        }
      }
    } else if($input['value'] == "0"){ /*solution to reset field*/
      $form_state->setValueForElement($element, "");
    }
  }

}
